"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dynamicIdentifierExtractor = exports.processTemplate = void 0;
const dotProp = require("dot-prop");
const entity_1 = require("./entity");
const decorator_error_1 = require("../errors/decorator.error");
const constants_1 = require("./constants");
function processTemplate(template, context) {
    let match;
    while ((match = constants_1.VAR_REGEXP.exec(template)) !== null) {
        if (match.index === constants_1.VAR_REGEXP.lastIndex) {
            constants_1.VAR_REGEXP.lastIndex++;
        }
        let [definition, varPath] = match;
        varPath = varPath.trim();
        if (!dotProp.has(context, varPath)) {
            throw new decorator_error_1.DecoratorError(`Unable to find ${varPath} property in Request from metadata field`);
        }
        let varValue = dotProp.get(context, varPath);
        if ((0, entity_1.isEntity)(varValue)) {
            varValue = (0, entity_1.toPlainDynamicIdentifier)(varValue);
        }
        template = template.replace(definition, varValue.toString());
    }
    return template;
}
exports.processTemplate = processTemplate;
function dynamicIdentifierExtractor(metadataField, hooks) {
    return (target, ctx) => {
        if (!Reflect.hasMetadata(metadataField, target)) {
            return null;
        }
        const { preHook, postHook } = hooks || {};
        let value = Reflect.getMetadata(metadataField, target);
        if (preHook) {
            value = preHook(value, ctx);
        }
        if (!ctx || typeof value !== 'string') {
            return value;
        }
        const request = ctx.switchToHttp().getRequest();
        value = processTemplate(value, request);
        if (postHook) {
            value = postHook(value, ctx);
        }
        return value;
    };
}
exports.dynamicIdentifierExtractor = dynamicIdentifierExtractor;
