"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Firewall = void 0;
const policy_1 = require("./interfaces/policy");
const matcher_1 = require("./matcher");
const global_storage_1 = require("./storages/global.storage");
const policy_vector_1 = require("./policy-vector");
const core_1 = require("./helpers/core");
class Firewall extends core_1.CoreHelper {
    constructor(storage, matcher) {
        super();
        this.storage = storage;
        this.matcher = matcher;
    }
    static create(storage, matcher) {
        return new this(storage || new global_storage_1.GlobalStorage(), matcher || new matcher_1.Matcher());
    }
    async isAllowed(rawResource, rawAction, rawPrincipal) {
        const resolution = await this.resolve(rawResource, rawAction, rawPrincipal);
        return resolution.allow.length > 0 && resolution.deny.length <= 0;
    }
    async isAllowedAny(rawResource, rawAction, rawPrincipal) {
        const resolution = await this.resolve(rawResource, rawAction, rawPrincipal);
        return resolution.allow.length > 0;
    }
    async isAllowedImplicit(rawResource, rawAction, rawPrincipal) {
        const resolution = await this.resolve(rawResource, rawAction, rawPrincipal);
        return resolution.deny.length <= 0;
    }
    async resolve(rawResource, rawAction, rawPrincipal) {
        const principal = (this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL));
        const rawPolicies = await this.storage.fetch(principal);
        const policies = policy_vector_1.PolicyVector.create(...rawPolicies);
        return this.matcher.match(rawResource, rawAction, principal, policies);
    }
}
exports.Firewall = Firewall;
