"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoreHelper = void 0;
const policy_1 = require("../interfaces/policy");
const entity_1 = require("../decorators/entity");
const wrong_policy_prop_format_error_1 = require("../errors/wrong-policy-prop-format.error");
class CoreHelper {
    static encode(policy, beatify = false) {
        return beatify
            ? JSON.stringify(policy, null, '  ')
            : JSON.stringify(policy);
    }
    static decode(rawPolicy) {
        return JSON.parse(rawPolicy);
    }
    isDynamicVector(x) {
        return Array.isArray(x);
    }
    isDynamicItem(x) {
        return (typeof x === 'object' &&
            typeof x
                .parse === 'function' &&
            'value' in x);
    }
    parseDynamicIdentifier(rawPolicy, prop, value) {
        const assignmentsVector = policy_1.DYNAMIC_IDENTIFIER_PROPS_MAPPING[prop];
        let valuesVector = value
            .split(policy_1.DELIMITER)
            .map((x) => x.trim())
            .map((x) => (x.length <= 0 ? CoreHelper.ANY : x));
        if (assignmentsVector.length < valuesVector.length) {
            throw new wrong_policy_prop_format_error_1.WrongPolicyPropFormat(rawPolicy, prop, assignmentsVector);
        }
        else if (valuesVector.length < assignmentsVector.length) {
            valuesVector = [
                ...valuesVector,
                ...new Array(assignmentsVector.length - valuesVector.length).fill(CoreHelper.ANY),
            ];
        }
        const identifierObject = {};
        for (const i in assignmentsVector) {
            const itemAssignment = assignmentsVector[i];
            const itemValue = valuesVector[i];
            identifierObject[itemAssignment] = itemValue;
        }
        return identifierObject;
    }
    normalizeDynamicIdentifier(thing, prop) {
        if (typeof thing === 'string') {
            return this.parseDynamicIdentifier({ [prop]: thing }, prop, thing);
        }
        return thing;
    }
    normalizeDynamicObject(thing, prop) {
        return (0, entity_1.isEntity)(thing)
            ? (0, entity_1.toDynamicIdentifier)(thing)
            : prop
                ? (this.normalizeDynamicIdentifier(thing, prop))
                : thing;
    }
}
exports.CoreHelper = CoreHelper;
CoreHelper.ANY = '*';
