"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IACryService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const firewall_1 = require("./firewall");
const sequelize_storage_1 = require("./storages/sequelize.storage");
const multiple_storage_1 = require("./storages/multiple.storage");
const global_storage_1 = require("./storages/global.storage");
const iacry_error_1 = require("./errors/iacry.error");
const matcher_1 = require("./matcher");
const constants_1 = require("./constants");
const core_1 = require("./helpers/core");
const ioredis_1 = require("./storages/cache/ioredis");
const cached_storage_1 = require("./storages/cached.storage");
const policy_manager_1 = require("./policy.manager");
let IACryService = class IACryService extends policy_manager_1.PolicyManager {
    constructor(options) {
        super(new multiple_storage_1.MultipleStorage());
        this.options = options;
        this.setup(options);
    }
    async isGranted(rawAction, rawPrincipal, rawResource = core_1.CoreHelper.ANY, rule = constants_1.IS_ALLOWED) {
        switch (rule) {
            case constants_1.IS_ALLOWED:
            case constants_1.IS_ALLOWED_ANY:
            case constants_1.IS_ALLOWED_IMPLICIT:
                break;
            default:
                throw new iacry_error_1.BaseError(`Unrecognized firewall rule: ${rule}`);
        }
        const resource = (this.normalizeDynamicObject(rawResource));
        const principal = (this.normalizeDynamicObject(rawPrincipal));
        return this.firewall[rule](resource, rawAction, principal);
    }
    setup(options) {
        if (options.storage) {
            switch (typeof options.storage) {
                case 'string':
                    switch (options.storage.toLowerCase()) {
                        case constants_1.SEQUELIZE_STORAGE:
                            this.storage.storages.push(new sequelize_storage_1.SequelizeStorage(options.storageRepository));
                            break;
                        default:
                            throw new iacry_error_1.BaseError(`Unrecognized PolicyInterface Storage type: ${options.storage}`);
                    }
                    break;
                default:
                    this.storage.storages.push(options.storage);
            }
        }
        if (options.policies) {
            this.storage.storages.push(new global_storage_1.GlobalStorage(options.policies));
        }
        if (options.cache) {
            let cache;
            switch (typeof options.cache) {
                case 'string':
                    switch (options.cache.toLowerCase()) {
                        case constants_1.IOREDIS_CACHE:
                            cache = new ioredis_1.IoRedis(options.cacheClient);
                            break;
                        default:
                            throw new iacry_error_1.BaseError(`Unrecognized PolicyCache type: ${options.cache}`);
                    }
                    break;
                default:
                    cache = options.cache;
            }
            this.setStorage(new cached_storage_1.CachedStorage(this.storage, cache, options.cacheOptions));
        }
        this.firewall = firewall_1.Firewall.create(this.storage, new matcher_1.Matcher(options.strict));
    }
};
IACryService = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(constants_1.IACRY_OPTIONS)),
    tslib_1.__metadata("design:paramtypes", [Object])
], IACryService);
exports.IACryService = IACryService;
