"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Matcher = void 0;
const micromatch = require("micromatch");
const policy_1 = require("./interfaces/policy");
const core_1 = require("./helpers/core");
class Matcher extends core_1.CoreHelper {
    constructor(strict = false) {
        super();
        this.strict = strict;
    }
    match(rawResource, rawAction, rawPrincipal, policies) {
        const resource = this.normalizeDynamicIdentifier(rawResource, policy_1.RESOURCE);
        const action = this.normalizeDynamicIdentifier(rawAction, policy_1.ACTION);
        const principal = this.normalizeDynamicIdentifier(rawPrincipal, policy_1.PRINCIPAL);
        let result = {
            allow: [],
            deny: [],
            abstain: [],
        };
        for (const policy of policies) {
            const resourceMatch = this.matchDynamicIdentifierVector(resource, policy.property(policy_1.RESOURCE), policy_1.RESOURCE);
            const actionMatch = this.matchDynamicIdentifierVector(action, policy.property(policy_1.ACTION), policy_1.ACTION);
            const principalMatch = this.matchDynamicIdentifierVector(principal, policy.property(policy_1.PRINCIPAL), policy_1.PRINCIPAL);
            if (resourceMatch && actionMatch && principalMatch) {
                switch (policy.Effect) {
                    case policy_1.Effect.ALLOW:
                        result.allow.push(policy);
                        break;
                    case policy_1.Effect.DENY:
                    default:
                        result.deny.push(policy);
                }
            }
            else {
                result.abstain.push(policy);
            }
        }
        return result;
    }
    matchDynamicIdentifierVector(source, targets, prop) {
        if (targets.length <= 0) {
            return true;
        }
        return (targets
            .map((target) => this.matchDynamicIdentifier(source, target, prop))
            .filter(Boolean).length > 0);
    }
    matchDynamicIdentifier(source, target, prop) {
        const matchProps = policy_1.DYNAMIC_IDENTIFIER_PROPS_MAPPING[prop];
        return (matchProps
            .map((key) => this.matchItem(source[key], target[key]))
            .filter(Boolean).length === matchProps.length);
    }
    matchItem(rawSource, rawTarget) {
        return micromatch.isMatch(this.normalizeValue(rawSource, true), this.normalizeValue(rawTarget)) || micromatch.isMatch(this.normalizeValue(rawTarget, true), this.normalizeValue(rawSource));
    }
    normalizeValue(rawValue, raw = false) {
        let value = rawValue.toString();
        if (!this.strict) {
            value = value.toLowerCase();
        }
        return raw ? value : value.replace(new RegExp(`\\${Matcher.ANY}+`), Matcher.ANY.repeat(2));
    }
}
exports.Matcher = Matcher;
