"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolicyManager = void 0;
const core_1 = require("./helpers/core");
const policy_1 = require("./interfaces/policy");
class PolicyManager extends core_1.CoreHelper {
    constructor(storage) {
        super();
        this.storage = storage;
    }
    setStorage(storage) {
        this.storage = storage;
        return this;
    }
    async attach(rawPrincipal, rawPolicies) {
        const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
        return this.storage.add(principal, rawPolicies);
    }
    async reset(rawPrincipal, rawPolicies) {
        const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
        return rawPolicies
            ? this.storage.save(principal, rawPolicies)
            : this.storage.purge(principal);
    }
    async retrieve(rawPrincipal) {
        const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
        const rawPolicies = await this.storage.fetch(principal);
        return rawPolicies.map((rawPolicy) => typeof rawPolicy === 'string' ? core_1.CoreHelper.decode(rawPolicy) : rawPolicy);
    }
    async retrieveBySid(sid, rawPrincipal) {
        const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
        const rawPolicies = await this.storage.fetchBySid(sid, principal);
        return rawPolicies.map((rawPolicy) => typeof rawPolicy === 'string' ? core_1.CoreHelper.decode(rawPolicy) : rawPolicy);
    }
    async upsertBySid(sid, rawPrincipal, rawPolicies) {
        const principal = (this.normalizeDynamicObject(rawPrincipal, policy_1.PRINCIPAL));
        return this.storage.saveBySid(sid, principal, rawPolicies);
    }
    async grant(rawAction, rawPrincipal, rawResource = core_1.CoreHelper.ANY, effect = policy_1.Effect.ALLOW, sid) {
        const resource = (this.normalizeDynamicObject(rawResource));
        const principal = (this.normalizeDynamicObject(rawPrincipal));
        const policy = {
            [policy_1.SID]: sid,
            [policy_1.EFFECT]: effect,
            [policy_1.ACTION]: rawAction,
            [policy_1.RESOURCE]: resource,
            [policy_1.PRINCIPAL]: principal,
        };
        return this.attach(rawPrincipal, [policy]);
    }
}
exports.PolicyManager = PolicyManager;
