"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultipleStorage = void 0;
class MultipleStorage {
    constructor(storages = []) {
        this.storages = storages;
        this.readonly = false;
    }
    async fetch(principal) {
        const result = await Promise.all(this.storages.map((storage) => storage.fetch(principal)));
        return result.reduce((accumulator, current) => {
            return accumulator.concat(...current);
        });
    }
    async save(principal, rawPolicies) {
        const result = await Promise.all(this.storages
            .filter((storage) => !storage.readonly)
            .map((storage) => storage.save(principal, rawPolicies)));
        return result.reduce((accumulator, current) => {
            return Math.min(accumulator, current);
        });
    }
    async add(principal, rawPolicies) {
        const result = await Promise.all(this.storages
            .filter((storage) => !storage.readonly)
            .map((storage) => storage.add(principal, rawPolicies)));
        return result.reduce((accumulator, current) => {
            return Math.min(accumulator, current);
        });
    }
    async purge(principal) {
        const result = await Promise.all(this.storages
            .filter((storage) => !storage.readonly)
            .map((storage) => storage.purge(principal)));
        return result.reduce((accumulator, current) => {
            return Math.max(accumulator, current);
        });
    }
    async fetchBySid(sid, principal) {
        const result = await Promise.all(this.storages.map((storage) => storage.fetchBySid(sid, principal)));
        return result.reduce((accumulator, current) => {
            return accumulator.concat(...current);
        });
    }
    async saveBySid(sid, principal, rawPolicies) {
        const result = await Promise.all(this.storages
            .filter((storage) => !storage.readonly)
            .map((storage) => storage.saveBySid(sid, principal, rawPolicies)));
        return result.reduce((accumulator, current) => {
            return Math.min(accumulator, current);
        });
    }
}
exports.MultipleStorage = MultipleStorage;
