"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoliciesStorage = void 0;
const tslib_1 = require("tslib");
const sequelize_typescript_1 = require("sequelize-typescript");
const sequelize_1 = require("sequelize");
const core_1 = require("../../helpers/core");
let PoliciesStorage = class PoliciesStorage extends sequelize_typescript_1.Model {
    static async savePrincipalPolicies(principal, rawPolicies, attach, sid) {
        if (!attach) {
            await this.destroyByPrincipal(principal, sid);
        }
        const result = await this.bulkCreate(rawPolicies.map((rawPolicy) => this.rawPolicyToPayload(principal, rawPolicy)));
        return result.length;
    }
    static async destroyByPrincipal(principal, sid) {
        const payload = {
            where: {
                entity: {
                    [sequelize_1.Op.or]: [null],
                },
                id: {
                    [sequelize_1.Op.or]: [null],
                },
                sid,
            },
        };
        if (principal.entity === core_1.CoreHelper.ANY) {
            delete payload.where.entity;
        }
        else {
            payload.where.entity[sequelize_1.Op.or].push(this.normalizePolicyItem(principal.entity));
        }
        if (principal.id === core_1.CoreHelper.ANY) {
            delete payload.where.id;
        }
        else {
            payload.where.id[sequelize_1.Op.or].push(principal.id);
        }
        if (!payload.where.entity && !payload.where.id && !payload.where.sid) {
            delete payload.where;
        }
        return this.destroy(payload);
    }
    static async findByPrincipal(principal, sid) {
        const payload = {
            where: {
                entity: {
                    [sequelize_1.Op.or]: [null],
                },
                id: {
                    [sequelize_1.Op.or]: [null],
                },
                sid,
            },
            attributes: ['policy'],
        };
        if (!sid) {
            delete payload.where.sid;
        }
        if (principal.entity === core_1.CoreHelper.ANY) {
            delete payload.where.entity;
        }
        else {
            payload.where.entity[sequelize_1.Op.or].push(this.normalizePolicyItem(principal.entity));
        }
        if (principal.id === core_1.CoreHelper.ANY) {
            delete payload.where.id;
        }
        else {
            payload.where.id[sequelize_1.Op.or].push(principal.id);
        }
        if (!payload.where.entity && !payload.where.id && !payload.where.sid) {
            delete payload.where;
        }
        const entries = await this.findAll(payload);
        return entries.map(({ policy }) => policy);
    }
    static rawPolicyToPayload(principal, rawPolicy) {
        const policy = typeof rawPolicy === 'string' ? core_1.CoreHelper.decode(rawPolicy) : rawPolicy;
        return {
            sid: policy.Sid || null,
            entity: !principal.entity || principal.entity === core_1.CoreHelper.ANY
                ? null
                : this.normalizePolicyItem(principal.entity),
            id: (!principal.id || principal.id) === core_1.CoreHelper.ANY
                ? null
                : principal.id,
            policy: core_1.CoreHelper.encode(policy),
        };
    }
    static normalizePolicyItem(item) {
        return item.toLowerCase();
    }
};
tslib_1.__decorate([
    sequelize_typescript_1.PrimaryKey,
    sequelize_typescript_1.AutoIncrement,
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", Number)
], PoliciesStorage.prototype, "pk", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.Index)('policy_sid'),
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "sid", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.Index)('policy_principal_id'),
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "id", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.Index)('policy_principal_entity'),
    (0, sequelize_typescript_1.AllowNull)(true),
    sequelize_typescript_1.Column,
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "entity", void 0);
tslib_1.__decorate([
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.BLOB),
    tslib_1.__metadata("design:type", String)
], PoliciesStorage.prototype, "policy", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.CreatedAt,
    tslib_1.__metadata("design:type", Date)
], PoliciesStorage.prototype, "createdAt", void 0);
tslib_1.__decorate([
    sequelize_typescript_1.UpdatedAt,
    tslib_1.__metadata("design:type", Date)
], PoliciesStorage.prototype, "updatedAt", void 0);
PoliciesStorage = tslib_1.__decorate([
    (0, sequelize_typescript_1.Table)({
        timestamps: true,
    })
], PoliciesStorage);
exports.PoliciesStorage = PoliciesStorage;
